red <- "<font color=\"red\">"
np <- 100
x <- c(0.5,rep(seq(0,1,length=np),np))
y <- c(0.5,rep(seq(0,1,length=np),each=np))
xy <- data.frame(x=x,y=y)

shinyServer(function(input, output) {
   
   output$checkGraph <- renderText({
      if(input$whichgraph=="base") return("")
      if(is.element("ggplot2", installed.packages()[,1])) {
          library(ggplot2)
          return("")
      }    
      else return(paste(red,"<h4>Warning: ggplot2 graphics package not found, go back to base graphics</h4>"))
      NULL       
   }) 
      
   vals <- reactiveValues(
    keeprows = c(FALSE,rep(FALSE, np*np))
  )
  
  output$text <- renderText({

    keep    <- xy[!vals$keeprows, , drop = FALSE]
    exclude <- xy[vals$keeprows, , drop = FALSE]
    
    if(nrow(exclude)<=1) return("<h5>We need at least two points!</h5>")
    line <- paste("<h5>","Correlation:",round(cor(exclude)[1,2],3),"</h5>")
    
    if(input$reg=="no") return(line)
    co <- round(coef(lm(exclude[,2]~exclude[,1])),3)
    sn <- ifelse(co[2]>0,"+","-")
    line[2] <- paste("<h5>Least Squares Regression Line:</h5>")
    line[3] <- paste("<h5>Y=",co[1],sn,abs(co[2]),"x</h5>")
    line
  })

  output$plot1 <- renderPlot({

    keep    <- xy[!vals$keeprows, , drop = FALSE]
    exclude <- xy[vals$keeprows, , drop = FALSE]
    
    if(input$whichgraph=="base") {
        plot(xy,type="n",xlab="",ylab="")
        points(exclude,pch=20,cex=1.3)
        if(nrow(exclude)>1) 
          if(input$reg=="yes") 
              abline(lm(exclude[,2]~exclude[,1]),lwd=2)
    }
    else {
       if(is.element("ggplot2", installed.packages()[,1])) {  
          plt <- ggplot(xy, aes(x, y)) +
              geom_point(data = exclude, shape = 20, fill = NA, color = "black", alpha = 1,size=3) +
              coord_cartesian(xlim = c(0, 1), ylim = c(0,1))+
              xlab("")+ylab("")    
          if(nrow(exclude)>1 & input$reg=="yes")
              print(plt+ stat_smooth(data=exclude,method = "lm",se=FALSE,size=1.2))
          else print(plt)     
       }   
    }   
  })

  observeEvent(input$plot1_click, {
    res <- nearPoints(xy, input$plot1_click, threshold = 5, 
          xvar="x",yvar="y",maxpoints = 1, allRows = TRUE)

    vals$keeprows <- xor(vals$keeprows, res$selected_)
  })


  # Reset all points
  observeEvent(input$exclude_reset, {
    vals$keeprows <- c(FALSE,rep(FALSE, np*np))
  })
   
 
  output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#correlation2\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 9/25/2015</h6>")     
  })   
  
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates Pearsons's Correlation Coefficient <br>
        Simply click inside the graph to get more points and observe the correlation coefficient</h5>"
        
  })
})

